#include "uTFT_ST7735.h"

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>

#define UP		0
#define DOWN	1
#define LEFT	2
#define RIGHT	3
#define SELECT	4
#define USED	5

#define EMPTY		0
#define PLAYER_X	1
#define PLAYER_O	2

unsigned char Three_in_a_RowX[8][3] = { //conditii de verificare castig pentru coordonata x
     { 0, 0, 0 },
     { 1, 1, 1 },
     { 2, 2, 2 },
     { 0, 1, 2 },
     { 0, 1, 2 },
     { 0, 1, 2 },
     { 0, 1, 2 },
     { 2, 1, 0 }};
unsigned char Three_in_a_RowY[8][3] = { //conditii de verificare castig pt coordonata y
     { 0, 1, 2 },
     { 0, 1, 2 },
     { 0, 1, 2 },
     { 0, 0, 0 },
     { 1, 1, 1 },
     { 2, 2, 2 },
     { 0, 1, 2 },
     { 0, 1, 2 }};

unsigned char key;
unsigned char selectX, selectY;
unsigned char current_player;

unsigned char bigMatrix[3][3], smallMatrix[9][9]; //matricile ce reprezinta jocul

ISR(PCINT0_vect)	//intreruperi butoane
{
    if(!(PINA & (1 << PA7)))
		key = UP;
    if(!(PINA & (1 << PA5)))
		key = DOWN;
    if(!(PINA & (1 << PA4)))
		key = LEFT;
    if(!(PINA & (1 << PA6)))
		key = RIGHT;
    if(!(PINA & (1 << PA3)))
		key = SELECT;
	_delay_ms(100);
}

void draw_X(){
	drawChar(5 + selectX *  14, 36 + selectY * 14, 'X', Color565(150, 150, 255), Color565(255, 255, 255));
}
void draw_O(){
	drawChar(5 + selectX *  14, 36 + selectY * 14, 'O', Color565(255, 150, 150), Color565(255, 255, 255));
}

unsigned char wonSmall(unsigned char x, unsigned char y, unsigned char current_player){ //verific daca castig intr-o matrice mica
	unsigned char i, j;
	unsigned char points;
	
	for(i = 0 ; i < 8 ; i++){
		points = 0;
		for(j = 0 ; j < 3 ; j++)
			if(smallMatrix[Three_in_a_RowX[i][j] + x][Three_in_a_RowY[i][j] + y] == current_player)
				points++;
		if(points == 3)
			return 1;
	}
	return 0;
}
unsigned char wonBig(unsigned char current_player){ //verific daca am castigat 3 matrici in linie
	unsigned char i, j;
	unsigned char points;
	
	for(i = 0 ; i < 8 ; i++){
		points = 0;
		for(j = 0 ; j < 3 ; j++)
			if(bigMatrix[Three_in_a_RowX[i][j]][Three_in_a_RowY[i][j]] == current_player)
				points++;
		if(points == 3)
			return 1;
	}
	return 0;
}

void outline(unsigned char x, unsigned char y){ //subliniaza casuta cutrenta
	uint16_t color;
	
	if(bigMatrix[selectX / 3][selectY / 3] == EMPTY)
		color = Color565(255, 255, 255);
	else if(bigMatrix[selectX / 3][selectY / 3] == PLAYER_X)	
		color = Color565(150, 150, 255);
	else
		color = Color565(255, 150, 150);
	
	drawFastVLine(2 + selectX * 14, 34 + selectY * 14, 11, color);
	drawFastVLine(12 + selectX * 14, 34 + selectY * 14, 11, color);
	drawFastHLine(2 + selectX * 14, 34 + selectY * 14, 11, color);
	drawFastHLine(2 + selectX * 14, 44 + selectY * 14, 11, color);
			
	selectX = x;
	selectY = y;
	
	drawFastVLine(2 + selectX * 14, 34 + selectY * 14, 11, Color565(0, 0, 0));
	drawFastVLine(12 + selectX * 14, 34 + selectY * 14, 11, Color565(0, 0, 0));
	drawFastHLine(2 + selectX * 14, 34 + selectY * 14, 11, Color565(0, 0, 0));
	drawFastHLine(2 + selectX * 14, 44 + selectY * 14, 11, Color565(0, 0, 0));
}

void main_init(){ //initializai ecran
	unsigned char i, j;
	
	init();
	fillScreen(Color565(255, 255, 255));
	setTextColor(Color565(0, 0, 0), Color565(255, 255, 255));

	DDRD |= (1 << PD7);
	PORTD |= (1 << PD7);
	
	DDRA &= ~(1 << PA7);
	PORTA |= (1 << PA7);
	DDRA &= ~(1 << PA6);
	PORTA |= (1 << PA6);
	DDRA &= ~(1 << PA5);
	PORTA |= (1 << PA5);
	DDRA &= ~(1 << PA4);
	PORTA |= (1 << PA4);
	DDRA &= ~(1 << PA3);
	PORTA |= (1 << PA3);
	
    PCICR  |= (1 << PCIE0);
    PCMSK0 |= (1 << PCINT7) | (1 << PCINT6) | (1 << PCINT5) | (1 << PCINT4) | (1 << PCINT3);
	
	selectX = 4; //initializari joc (pozitie de inceput, casute goale)
	selectY = 4;
	current_player = PLAYER_X; 
	outline(selectX, selectY);
	key = USED;
	
	for(i = 0 ; i < 9 ; i++)
		for(j = 0 ; j < 9 ; j++)
			smallMatrix[i][j] = EMPTY;
	for(i = 0 ; i < 3 ; i++)
		for(j = 0 ; j < 3 ; j++)
			bigMatrix[i][j] = EMPTY;
	sei();
}


void draw_grid(){ //desenare chenar joc
	unsigned char i;
	for(i = 0 ; i < 10 ; i++){
		drawFastVLine(i * 14, 32, 127, Color565(0, 0, 0));
		if(!(i % 3)){
			drawFastVLine(i * 14 - 1, 32, 127, Color565(0, 0, 0));
			drawFastVLine(i * 14 + 1, 32, 127, Color565(0, 0, 0));
		}
	}
	for(i = 0 ; i < 10 ; i++){
		drawFastHLine(0, 32 + i * 14, 127, Color565(0, 0, 0));
		if(!(i % 3)){
			drawFastHLine(0, 32 + i * 14 - 1, 127, Color565(0, 0, 0));
			drawFastHLine(0, 32 + i * 14 + 1, 127, Color565(0, 0, 0));
		}
	}
}

unsigned char gameover(unsigned char player){ //verificare conditie de castig joc si aifsare mesaj corespunzator
	unsigned char won = wonBig(player);
	if(won && player == PLAYER_O){
		print("\nPlayer O won !!!!");
		fillRect(2, 34, 123, 123, Color565(255, 150, 150));
	}
	if(won && player == PLAYER_X){
		print("\nPlayer X won !!!!");
		fillRect(2, 34, 123, 123, Color565(150, 150, 255));
	}
	return won;
}
void fillBigMatrix(unsigned char current_player){ //umblere spatiu de joc cu culoarea castigatoare
	unsigned char beginIndexX, beginIndexY;
	beginIndexX = selectX / 3 * 3;
	beginIndexY = selectY / 3 * 3;
	if(wonSmall(beginIndexX, beginIndexY, current_player)){
		bigMatrix[selectX / 3][selectY / 3] = current_player;
		if(current_player == PLAYER_O)
			fillRect(beginIndexX * 14 + 2, beginIndexY * 14 + 34, 39, 39, Color565(255, 150, 150));
		else
			fillRect(beginIndexX * 14 + 2, beginIndexY * 14 + 34, 39, 39, Color565(150, 150, 255));
	}
}

unsigned char invalid(unsigned char prevX, unsigned char prevY){ //verificare mutare nevalide
	unsigned char destX = (prevX % 3) * 3, destY = (prevY % 3) * 3;
	
	if(prevX == 11) //daca exista deja caracter
		return 0;
	if(bigMatrix[destX / 3][destY / 3] != EMPTY) //daca matricea e castigata
		return 0;
	if(selectX >= destX && selectX < destX + 3 &&
	   selectY >= destY && selectY < destY + 3) //daca se afla in matricea corespunzatoare
		return 0;
	return 1;
}

void main_loop()
{
	unsigned char prevX = 11, prevY = 11;
	draw_grid();

    for(;;)
    {
		switch(key){
			case UP:
				if(selectY > 0)
					outline(selectX, selectY - 1);
				key = USED;
				break;
			case DOWN:
				if(selectY < 8)
					outline(selectX, selectY + 1);
				key = USED;
				break;
			case LEFT:
				if(selectX > 0)
					outline(selectX - 1, selectY);
				key = USED;
				break;
			case RIGHT:
				if(selectX < 8)
					outline(selectX + 1, selectY);
				key = USED;
				break;
			case SELECT:
				if(bigMatrix[selectX / 3][selectY / 3] != EMPTY || invalid(prevX, prevY)){
					key = USED;
					continue;
				}
				if(smallMatrix[selectX][selectY] == EMPTY &&
				   bigMatrix[selectX / 3][selectY / 3] == EMPTY){ //daca se poate pune in casuta respectiva
					prevX = selectX; prevY = selectY;
					smallMatrix[selectX][selectY] = current_player;
					if(current_player == PLAYER_X){ //verificare player curent
						current_player = PLAYER_O;
						draw_X();
					} else {
						current_player = PLAYER_X;
						draw_O();
					}
					fillBigMatrix(smallMatrix[selectX][selectY]);
					if(gameover(smallMatrix[selectX][selectY]))
						return;
				}
				key = USED;
				break;
			case USED:
				_delay_ms(50);
				continue;
		}
    }
}



int main()
{
	main_init();
	main_loop();
    return 0;
}